/***************************************************************************
                          buy.cpp  -  description
                             -------------------
    begin                : Tue Jun 5 2001
    copyright            : (C) 2001 by Jess David Daz Leal
    email                : davileal@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#define _GNU_SOURCE   // This is for asprintf. It has an effect on stdio.h

#include "buy.h"

#include <qlayout.h>
#include <qvbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qaccel.h>
#include <qmessagebox.h>
#include <qvalidator.h>

#include <stdio.h>
#include <stdlib.h> // To use the 'exit()' function.


extern ShopDB* theShopDB;


Buy::Buy(QWidget* parent, const char* name) : QWidget( parent, name )
{
	QGridLayout* grid = new QGridLayout( this, 1, 3 );
	grid->setMargin( SPACING );
	grid->setSpacing( SPACING );

	QVBox* vbox0 = new QVBox( this, "vbox0" );
	grid->addWidget( vbox0, 0, 0 ); 
	grid->setColStretch( 0, 40 );

  	QLabel* labelBought= new QLabel( "Product          Units      Euros", vbox0, "labelBought" );
  	labelBought->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	lbBoughtProds = new QListBox( vbox0, "lbBoughtProds" );
  	lbBoughtProds->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	lbBoughtProds->insertItem( QString(MAX__SHORT_DSC) + QString(" ") + QString(MAX_UNITS) +
				 QString(" ") + QString(MAX_PRICE__CURRENCY1) );
  	lbBoughtProds->setMinimumWidth( lbBoughtProds->maxItemWidth() + SPACE_TO_SCROLLBAR );
  	lbBoughtProds->setMinimumHeight( 420 );
  	lbBoughtProds->removeItem( 0 );

	QHBox* hbox0 = new QHBox( vbox0, "hbox0" );
	hbox0->setSpacing(SPACING);

  	QLabel* lbTotal = new QLabel( "Total", hbox0, "lbTotal" );
  	lbTotal->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	totalBought = new QLineEdit( hbox0, "leTotalBought" );
  	totalBought->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	totalBought->setReadOnly(true);
  	totalBought->setAlignment( Qt::AlignRight );

	QWidget* wSpace = new QWidget( hbox0, "wSpace" );
  	wSpace->setMinimumWidth( SPACE_TO_SCROLLBAR - SPACING - 2 );

	QWidget* wSpace2 = new QWidget( vbox0, "wSpaces" );
  	wSpace2->setMinimumHeight(20);

  	QPushButton* remove = new QPushButton( "Remove", vbox0, "pbRemove" );
  	remove->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );
  	connect( remove, SIGNAL(clicked()), SLOT(remIfSelected()) );


	QVBox* vbox1 = new QVBox( this, "vbox1" );
	grid->addWidget( vbox1, 0, 1 ); 
	grid->setColStretch( 1, 30 );

	QHBox* hbox1 = new QHBox( vbox1, "hbox1" );

	QVBox* vbox10 = new QVBox( hbox1, "vbox10" );
	QVBox* vbox11 = new QVBox( hbox1, "vbox11" );

  	QLabel* labelCod= new QLabel( "Cod", vbox10, "labelCod" );
  	labelCod->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	cod = new QLineEdit( vbox10, "leCod" );
  	cod->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
  	cod->setAlignment( Qt::AlignRight );
	QIntValidator* validator = new QIntValidator(this);
	cod->setValidator(validator);
  	connect( cod, SIGNAL(textChanged(const QString &)), SLOT(selectProd()) );

  	QLabel* labelDsc= new QLabel( "Description", vbox11, "labelDsc" );
  	labelDsc->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	desc = new QLineEdit( vbox11, "leDesc" );
  	desc->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
  	desc->setReadOnly(true);
      	desc->setMinimumWidth( 130 + 4 ); //XXX-Davi: No adjust to different fonts
      	desc->setMaximumWidth( 130 + 4 );


  	lbProdsToBuy = new QListBox( vbox1, "lbProdsToBuy" );
  	lbProdsToBuy->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );
  	this->setFocusProxy(lbProdsToBuy);
  	connect( lbProdsToBuy, SIGNAL(currentChanged(QListBoxItem*)), SLOT(prodSelected()) );

	// This method have to be called only once due to it make up 'prodsToBuy = new ProdsToBuy[n+1];'
	// and we delete it into the destructor of this class.
  	prodsToBuy = theShopDB->createProdsToBuy();
  	// We will free the prodsToBuy array at salepoint destruction

	// Get the most long product code
	TheMostLongProdCode = 0;
  	if( prodsToBuy!=NULL && prodsToBuy[0].shortdsc != NULL )
	{
  		int i;
		for( i=0; prodsToBuy[i].shortdsc != NULL; i++ )
		{
			char* tmp;
			asprintf(&tmp, "%d", prodsToBuy[i].code);
			if ( strlen(tmp) > TheMostLongProdCode )
				TheMostLongProdCode = strlen(tmp);
			free(tmp);
  		}

		// Load the list of products to sell into the GUI
  		for( i=0; prodsToBuy[i].shortdsc != NULL; i++ )
  		{
  			char* tmp;
  			asprintf( &tmp, "%d  %s", prodsToBuy[i].code, prodsToBuy[i].shortdsc );

			QString tmp2;
			tmp2="";
			int l;
			l = strlen(tmp);
			int j;
			for( j=0; j+l<TheMostLongProdCode+2+strlen(MAX__SHORT_DSC); j++ )
			{
				tmp2 = " " + tmp2;
			}
  			lbProdsToBuy->insertItem( tmp2 + QString(tmp) );
  		}
  	}
  	else QMessageBox::warning( 0, "Data Base error",
				   QString("The report_ProdsToBuy() ShopDB method fail. Maybe it \n") +
				   QString("is due to wrong Data Base installation or the table 'products' \n") +
				   QString("is empty. \n\n") + QString(theShopDB->error) );

  	lbProdsToBuy->setMinimumWidth( lbProdsToBuy->maxItemWidth() + SPACE_TO_SCROLLBAR );


  	// XXX-Davi: (XXX-QT/KDE-expert)
  	// Client requirement: When a key is pressed the list of products to sell
  	// select and show the first product whose first letter of the 'short description'
  	// is equal to the letter pressed.
  	// NNNote: Not only when lbProdsToBuy widget has the focus but when any
  	//         widget is activated.


	const int EDIT=88;
	const int SPIN=61;
	QWidget* wid = new QWidget( this, "wid" );
  	wid->setMinimumWidth(5+SPIN+EDIT+EDIT+5);
	grid->addWidget( wid, 0, 2 ); 
	grid->setColStretch( 2, 30 );

  	QLabel* labelEuro = new QLabel( "Euros", wid, "labelEuro");
	labelEuro->setGeometry( QRect(0+SPIN+SPACING, 0, EDIT, 30) );
  	labelEuro->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	previousValue = "0.00";
  	totalC1 = new QLineEdit( wid, "leTotal" );
  	totalC1->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
	doubleValidator = new QDoubleValidator(this);
	doubleValidator->setRange(0.00,99999.99,2); // XXX-Davi: Check support to select among several currencies //XXX-Davi: Use const. Do we use convertion to char or to float?, Uhmmm ...
	totalC1->setValidator(doubleValidator);
  	totalC1->setAlignment( Qt::AlignRight );
  	totalC1->setGeometry( QRect(SPIN+SPACING, 70, EDIT, 30) );
  	connect( totalC1, SIGNAL(textChanged(const QString &)), SLOT(checkTotalFormat()) );

  	units = new QSpinBox( wid, "sbUnits" );
  	units->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	units->setValue(1);
  	units->setMinValue(1);
  	units->setMaxValue(999);
  	units->setGeometry( QRect(5, 70, SPIN, 30) );

  	QPushButton* bought = new QPushButton( "Bought", wid, "pbBought" );
  	bought->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_1, QFont::Normal ) );
  	connect( bought, SIGNAL(clicked()), SLOT(boughtProduct()) );
  	bought->setGeometry( QRect(5, 130, SPIN+EDIT+EDIT, 60) );

  	QPushButton* save = new QPushButton( "Save", wid, "pbSave" );
  	save->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_1, QFont::Normal ) );
  	connect( save, SIGNAL(clicked()), SLOT(savePurchasedProds()) );
  	save->setGeometry( QRect(58, 300, 150, 60) );


  	QAccel* accel = new QAccel( this );
  	accel->connectItem( accel->insertItem( Key_Enter ), this, SLOT(boughtProduct()) );
  	accel->connectItem( accel->insertItem( Key_Return ),this, SLOT(boughtProduct()) );
}

void Buy::remIfSelected()
{
	if ( lbBoughtProds->isSelected(lbBoughtProds->currentItem()) )
	{
		lbBoughtProds->removeItem( lbBoughtProds->currentItem() );
		reEvaluateBoughtProdTotal();
	}
}

void Buy::boughtProduct()
{
	// Add the new row
	int i, l;
	QString tmp, result;

	tmp = desc->text();
	l = tmp.length();
	for( i=l; i<strlen(MAX__SHORT_DSC); i++ )
		tmp.append( QString(" ") );
	result = tmp;

	tmp = units->text();
	l = tmp.length();
	for( i=l; i<strlen(MAX_UNITS); i++ )
		tmp.insert( 0, QString(" ") );
	result = result + " " + tmp;

	tmp = totalC1->text();
	l = tmp.length();
	for( i=l; i<strlen(MAX_PRICE__CURRENCY1); i++ )
		tmp.insert( 0, QString(" ") );
	result = result + " " + tmp;

	lbBoughtProds->insertItem( result );

	// Reevaluate the Total
	reEvaluateBoughtProdTotal();
}

void Buy::reEvaluateBoughtProdTotal()
{
	float euros=0.0;
	unsigned int j;
	for( j=0; j<lbBoughtProds->count(); j++ )
	{
		euros += ( (lbBoughtProds->text(j)).right(strlen(MAX_PRICE__CURRENCY1)) ).toFloat(); //XXX-Davi: Is this operation accuracy?
		// The overflow does not work with this. So, we must
		// find the last word or be sure to avoid the overflow.
		// We now are sure to avoid the overflow from the
		// cause ('totalC1' product total).
	}
	char* tmp;
	asprintf(&tmp, "%.2f", euros );
	totalBought->setText( QString(tmp) );
	free(tmp);
}

void Buy::reEvaluateTotal()
{
	unsigned int i_units;

	i_units = (units->text()).toUInt();

	QString qTotalC1 = totalC1->text();
	/*unsigned*/ float f_totalC1;
	f_totalC1 = qTotalC1.toFloat();  //XXX-Davi: Check the accuracy of this operation

	char* Currency1;
	asprintf(&Currency1, "%.2f", i_units*f_totalC1 );
  	// If there is overflow then send a window-warning to the user to call to the support.
	// Check the overflow: Workaround: Avoid the overflow
	if( strlen(Currency1) > strlen(MAX_PRICE__CURRENCY1) ) 
	{
		units->setValue(1);
//XXX-Davi: This is commented to avoid a new bug.
//  		QMessageBox::information( this, "", "Please, split the units to avoid overflow. For example use, 5 and 5 instead of 10." );
//  		//units->stepDown();
  		qWarning("Please, split the units to avoid overflow. For example use, 5 and 5 instead of 10." );
	}else{
		totalC1->setText( QString(Currency1) );
		free(Currency1);
	}
}

void Buy::prodSelected()
{
	cod->setText( (lbProdsToBuy->currentText()).left(TheMostLongProdCode) );
	desc->setText( (lbProdsToBuy->currentText()).right(strlen(MAX__SHORT_DSC)) );

	units->setValue(1);
	totalC1->setText("0.00");
}

void Buy::savePurchasedProds()
{
	//(XXX-Tomasz)  The products which have been bought are kept in the "PurchasesSalesHistory"
	//              table, filling all the columns of the table. The transactionId is set
	//              in accordance.

	//(XXX-Tomasz)  The products which have been bought are added to the "Stock"
	//              in the units according with the units bought


	// Remove all items from lbBoughtProds list box
	unsigned int i, j;
	j = lbBoughtProds->count();
	for( i=0; i<j; i++ )
		lbBoughtProds->removeItem( j -i -1 );
		
	
	reEvaluateBoughtProdTotal();
}

void Buy::selectProd()
{
	// If there is, it finds the product with this code into the ListBox and selects it.

	// Make up de 'code' patern
	int i, l;
	QString code;

	code = cod->text();
	l = code.length();
	for( i=l; i<TheMostLongProdCode; i++ )
		code.insert( 0, QString(" ") );

	// Look for it. Find exact mach of code
	int j;
	j = lbProdsToBuy->count();
	for( i=0; i<j && !( (lbProdsToBuy->text(i)).left(TheMostLongProdCode) ).contains(code) ; i++ )  ;

	// Select if there is
	if (i<j)
	{
		if( !lbProdsToBuy->isSelected(i) )  // To avoid an infinite loop.
			lbProdsToBuy->setSelected(i,true);
		lbProdsToBuy->ensureCurrentVisible();
	}
}

void Buy::checkTotalFormat()
{
	//XXX-Davi: Does not work:   doubleValidator->validate(totalC1->text());
}

Buy::~Buy()
{
	delete prodsToBuy;
}
