/***************************************************************************
                          fitInventory.cpp  -  description
                             -------------------
    begin                : Tue Jun 5 2001
    copyright            : (C) 2001 by Jess David Daz Leal
    email                : davileal@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#define _GNU_SOURCE   // This is for asprintf. It has an effect on stdio.h

#include "fitInventory.h"

#include <qlayout.h>
#include <qvbox.h>
#include "qlineedit.h"
#include "qlabel.h"
#include "qlistbox.h"
#include "qpushbutton.h"
#include <qmessagebox.h>
#include <qvalidator.h>


extern ShopDB* theShopDB;


FitInventory::FitInventory(QWidget* parent, const char* name) : QWidget ( parent, name )
{
	QGridLayout* grid = new QGridLayout( this, 2, 3 );
	grid->setMargin( SPACING );
	grid->setSpacing( SPACING );


	QVBox* vbox00 = new QVBox( this, "vbox00" );
	grid->addWidget( vbox00, 0, 0 ); 

	QHBox* hbox00_0 = new QHBox( vbox00, "hbox00_0" );

	QVBox* vbox00_00 = new QVBox( hbox00_0, "vbox00_00" );
	QVBox* vbox00_01 = new QVBox( hbox00_0, "vbox00_01" );

  	QLabel* labelCod= new QLabel( "Cod", vbox00_00, "labelCod" );
  	labelCod->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	cod = new QLineEdit( vbox00_00, "leCod" );
  	cod->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
  	cod->setAlignment( Qt::AlignRight );
	QIntValidator* intValidator = new QIntValidator(this);
	cod->setValidator(intValidator);
  	connect( cod, SIGNAL(textChanged(const QString &)), SLOT(selectProd()) );

  	QLabel* labelDsc= new QLabel( "Description", vbox00_01, "labelDsc" );
  	labelDsc->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	desc = new QLineEdit( vbox00_01, "leDesc" );
  	desc->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
  	desc->setReadOnly(true);
      	desc->setMinimumWidth( 130 + 4 ); //XXX-Davi: No adjust to different fonts
      	desc->setMaximumWidth( 130 + 4 );
	//desc->XXX-Davi: set upercase. Mainly to make it easier typing a new product.

  	lbProds = new QListBox( vbox00, "lbProds" );
  	lbProds->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );
  	this->setFocusProxy(lbProds);
  	connect( lbProds, SIGNAL(currentChanged(QListBoxItem*)), SLOT(prodSelected()) );

	// This method have to be called only once due to it make up 'prodsToBuy = new ProdsToBuy[n+1];'
	// and we delete it into the destructor of this class.
  	prodsToBuy = theShopDB->createProdsToBuy();
  	// We will free the prodsToBuy array at salepoint destruction

	// Get the most long product code
	TheMostLongProdCode = 0;
  	if( prodsToBuy!=NULL && prodsToBuy[0].shortdsc != NULL )
	{
  		int i;
		for( i=0; prodsToBuy[i].shortdsc != NULL; i++ )
		{
			char* tmp;
			asprintf (&tmp, "%d", prodsToBuy[i].code);
			if ( strlen(tmp) > TheMostLongProdCode )
				TheMostLongProdCode = strlen(tmp);
			free (tmp);
  		}

		// Load the list of products to sell into the GUI
  		for( i=0; prodsToBuy[i].shortdsc != NULL; i++ )
  		{
  			char* tmp;
  			asprintf( &tmp, "%d  %s", prodsToBuy[i].code, prodsToBuy[i].shortdsc );

			QString tmp2;
			tmp2="";
			int l;
			l = strlen(tmp);
			int j;
			for( j=0; j+l<TheMostLongProdCode+2+strlen(MAX__SHORT_DSC); j++ )
			{
				tmp2 = " " + tmp2;
			}
  			lbProds->insertItem( tmp2 + QString(tmp) );
  		}
  	}
  	else QMessageBox::warning( 0, "Data Base error",
				   QString("The report_ProdsToBuy() ShopDB method fail. Maybe it \n") +
				   QString("is due to wrong Data Base installation or the table 'products' \n") +
				   QString("is empty. \n\n") + QString(theShopDB->error) );

  	lbProds->setMinimumWidth( lbProds->maxItemWidth() + SPACE_TO_SCROLLBAR );


	QVBox* vbox01 = new QVBox( this, "vbox01" );
	grid->addWidget( vbox01, 0, 1 ); 

	QLabel* labelLongDsc = new QLabel( "Description", vbox01, "labelLongDsc" );
  	labelLongDsc->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );
	
	longDsc = new QLineEdit( vbox01, "longDsc" );
	longDsc->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
	longDsc->setReadOnly(true);
	longDsc->setMinimumWidth( 300 );  //XXX-Davi:  Must it use a constant?

	QWidget* vBoxSpace01 = new QWidget( vbox01, "vBoxSpace01" );
	vBoxSpace01->setMinimumHeight( 485 );  //XXX-Davi:  Must it use a constant?


	QVBox* vbox02 = new QVBox( this, "vbox02" );
	grid->addWidget( vbox02, 0, 2 ); 

	QHBox* hbox02_0 = new QHBox( vbox02, "hbox02_0" );

	QVBox* vbox02_00 = new QVBox( hbox02_0, "vbox02_00" );
	QVBox* vbox02_01 = new QVBox( hbox02_0, "vbox02_01" );
	QVBox* vbox02_02 = new QVBox( hbox02_0, "vbox02_02" );

	QLabel* labelUnits = new QLabel( "Units", vbox02_00, "labelUnits" );
	labelUnits->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );
	virtualUnits = new QLineEdit( vbox02_00, "virtualUnits" );
	virtualUnits->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
	//XXX-Davi: Define the supported format   "999.999"
	virtualUnits->setReadOnly(true);

  	QLabel* labelRealUnits = new QLabel( "Real units", vbox02_01, "labelRealUnits" );
  	labelRealUnits->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );
	QLineEdit* realUnits = new QLineEdit( vbox02_01, "realUnits" );
	realUnits->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	

  	QLabel* labelDeviation = new QLabel( "Deviation", vbox02_02, "labelDeviation" );
  	labelDeviation->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );
	QLineEdit* deviation = new QLineEdit( vbox02_02, "deviation" );
	deviation->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
	deviation->setReadOnly(true);

	// Fit the showed product
	///XXX-Davi: Accel via Intro and Enter keys
	QPushButton* fit = new QPushButton( "Fit this product", vbox02, "fit" );
	fit->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );
	connect( fit, SIGNAL(clicked()), SLOT(fitProd()) );

	QWidget* vBoxSpace02 = new QWidget( vbox02, "vBoxSpace02" );
  	vBoxSpace02->setMinimumHeight( 350 );  //XXX-Davi:  Must it use a constant?

	QPushButton* show = new QPushButton( "Show changes", vbox02, "show" );
	show->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	QPushButton* trash = new QPushButton( "Trash changes", vbox02, "trash" );
	trash->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	QPushButton* accept = new QPushButton( "Accept the inventory", vbox02, "accept" );
	accept->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );
}

void FitInventory::prodSelected()
{
	cod->setText( (lbProds->currentText()).left(TheMostLongProdCode) );
	desc->setText( (lbProds->currentText()).right(strlen(MAX__SHORT_DSC)) );

	longDsc->setText( QString(prodsToBuy[lbProds->currentItem()].longdsc) );

	char* stock;
	unsigned int tmp_stock = prodsToBuy[lbProds->currentItem()].stock; //XXX-Davi: Does it fail due to the List Box is now sorted by shordsc. I do not think so due to it is sorted in the query 'ORDER BY'.
	asprintf(&stock, "%d", tmp_stock ); //XXX-Davi: Use QString
	virtualUnits->setText( QString(stock) );
	free(stock);
}

void FitInventory::selectProd()
{
	// If there is, it finds the product with this code into the ListBox and selects it.

	// Make up de 'code' patern
	int i, l;
	QString code;

	code = cod->text();
	l = code.length();
	for( i=l; i<TheMostLongProdCode; i++ )
		code.insert( 0, QString(" ") );

	// Look for it. Find exact mach of code
	int j;
	j = lbProds->count();
	for( i=0; i<j && !( (lbProds->text(i)).left(TheMostLongProdCode) ).contains(code) ; i++ )  ;

	// Select if there is
	if (i<j)
	{
		if( !lbProds->isSelected(i) )  // To avoid an infinite loop.
			lbProds->setSelected(i,true);
		lbProds->ensureCurrentVisible();
	}
}

void FitInventory::fitProd()
{
}

FitInventory::~FitInventory()
{
}
