/***************************************************************************
                          prodsManage.cpp  -  description
                             -------------------
    begin                : Tue Jun 5 2001
    copyright            : (C) 2001 by Jess David Daz Leal
    email                : davileal@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#define _GNU_SOURCE   // This is for asprintf. It has an effect on stdio.h

#include "prodsManage.h"

#include <qlayout.h>
#include "qvbox.h"
#include "qlineedit.h"
#include "qlabel.h"
#include "qpushbutton.h"
#include <qmessagebox.h>
#include <qvalidator.h>

#include <stdio.h>


extern ShopDB* theShopDB;


ProdsManage::ProdsManage(QWidget* parent, const char* name) : QWidget( parent, name )
{
	QGridLayout* grid = new QGridLayout( this, 2, 2 );
	grid->setMargin( SPACING );
	grid->setSpacing( SPACING );


	QVBox* vbox00 = new QVBox( this, "vbox00" );
	grid->addWidget( vbox00, 0, 0 ); 

	QHBox* hbox00_0 = new QHBox( vbox00, "hbox00_0" );

	QVBox* vbox00_00 = new QVBox( hbox00_0, "vbox00_00" );
	QVBox* vbox00_01 = new QVBox( hbox00_0, "vbox00_01" );

  	QLabel* labelCod0= new QLabel( "Cod", vbox00_00, "labelCod0" );
  	labelCod0->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	cod0 = new QLineEdit( vbox00_00, "leCod0" );
  	cod0->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
  	cod0->setAlignment( Qt::AlignRight );
	QIntValidator* intValidator = new QIntValidator(this);
	cod0->setValidator(intValidator);
	//XXXXX-Davi:  cod0->setBackgroundColor( QColor(255,0,0) );
	//XXXX-Davi:   cod0->setPalette( QPalette( QColor(80, 255, 80) ) );
	//XXXX-Davi:   cod0->setBackgroundMode( QWidget::PaletteHighlightedText );
	cod0->setBackgroundMode( QWidget::PaletteDark ); //XXX-Davi: XXX-QT/KDE-expert: This does not work
  	connect( cod0, SIGNAL(textChanged(const QString &)), SLOT(selectProdToSell()) );

  	QLabel* labelDsc0= new QLabel( "Description", vbox00_01, "labelDsc0" );
  	labelDsc0->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	desc0 = new QLineEdit( vbox00_01, "leDesc0" );
  	desc0->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
  	desc0->setReadOnly(true);
      	desc0->setMinimumWidth( 130 + 4 ); //XXX-Davi: No adjust to different fonts
      	desc0->setMaximumWidth( 130 + 4 );
	//desc0->XXX-Davi: set upercase. Mainly to make it easier typing a new product.
  	connect( desc0, SIGNAL(textChanged(const QString &)), SLOT(prodToSellNotSaved()) );

  	lbProdsToSell = new QListBox( vbox00, "lbProdsToSell" );
  	lbProdsToSell->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );
  	this->setFocusProxy(lbProdsToSell);
  	connect( lbProdsToSell, SIGNAL(currentChanged(QListBoxItem*)), SLOT(prodToSellSelected()) );

	// This method have to be called only once due to it make up 'prodsToSell = new ProdsToSell[n+1];'
	// and we delete it into the destructor of this class.
  	prodsToSell = theShopDB->createProdsToSell();
  	// We will free the prodsToSell array at salepoint destruction

	// Get the most long product code
	TheMostLongToSellProdCode = 0;
  	if( prodsToSell!=NULL && prodsToSell[0].shortdsc != NULL )
	{
  		int i;
		for( i=0; prodsToSell[i].shortdsc != NULL; i++ )
		{
			char* tmp;
			asprintf(&tmp, "%d", prodsToSell[i].code);
			if ( strlen(tmp) > TheMostLongToSellProdCode )
				TheMostLongToSellProdCode = strlen(tmp);
			free(tmp);
  		}

		// Load the list of products to sell into the GUI
  		for( i=0; prodsToSell[i].shortdsc != NULL; i++ )
  		{
  			char* tmp;
  			asprintf( &tmp, "%d  %s", prodsToSell[i].code, prodsToSell[i].shortdsc );

			QString tmp2;
			tmp2="";
			int l;
			l = strlen(tmp);
			int j;
			for( j=0; j+l<TheMostLongToSellProdCode+2+strlen(MAX__SHORT_DSC); j++ )
			{
				tmp2 = " " + tmp2;
			}
  			lbProdsToSell->insertItem( tmp2 + QString(tmp) );
  		}
  	}
  	else QMessageBox::warning( 0, "Data Base error",
				   QString("The report_ProdsToSell() ShopDB method fail. Maybe it \n") +
				   QString("is due to wrong Data Base installation or the table 'products' \n") +
				   QString("is empty. \n\n") + QString(theShopDB->error) );

  	lbProdsToSell->setMinimumWidth( lbProdsToSell->maxItemWidth() + SPACE_TO_SCROLLBAR );

	QHBox* hbox00_1 = new QHBox( vbox00, "hbox00_1" );

	prodToSellNew = new QPushButton( "New", hbox00_1, "prodToSellNew" );
	prodToSellNew->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );
  	connect( prodToSellNew, SIGNAL(clicked()), SLOT(newToSellProd()) );

	prodToSellRem = new QPushButton( "Remove", hbox00_1, "prodToSellRem" );
	prodToSellRem->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );
  	connect( prodToSellRem, SIGNAL(clicked()), SLOT(removeToSellProd()) );

	prodToSellSave = new QPushButton( "Save", hbox00_1, "prodToSellSave" );
	prodToSellSave->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );
	prodToSellSave->setEnabled(false);
  	connect( prodToSellSave, SIGNAL(clicked()), SLOT(saveToSellProd()) );


	QVBox* vbox10 = new QVBox( this, "vbox10" );
	grid->addWidget( vbox10, 0, 1 ); 

	QHBox* hbox10_0 = new QHBox( vbox10, "hbox10_0" );

	QVBox* vbox10_00 = new QVBox( hbox10_0, "vbox10_00" );
	QVBox* vbox10_01 = new QVBox( hbox10_0, "vbox10_01" );

  	QLabel* labelLongDsc0= new QLabel( "Description", vbox10_00, "labelLongDsc0" );
  	labelLongDsc0->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	prodToSellLongDsc = new QLineEdit( vbox10_00);
	prodToSellLongDsc->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	connect( prodToSellLongDsc, SIGNAL(textChanged(const QString &)), SLOT(prodToSellNotSaved()) );

  	QLabel* labelUnitPrice0= new QLabel( "Unit price", vbox10_01, "labelUnitPrice0" );
  	labelUnitPrice0->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	prodToSellUnitPrice = new QLineEdit( vbox10_01 );
	prodToSellUnitPrice->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	prodToSellUnitPrice->setAlignment( Qt::AlignRight );
	QDoubleValidator* doubleValidator = new QDoubleValidator(this);
	prodToSellUnitPrice->setValidator(doubleValidator);
  	connect( prodToSellUnitPrice, SIGNAL(textChanged(const QString &)), SLOT(prodToSellNotSaved()) );


	QHBox* hbox10_1 = new QHBox( vbox10, "hbox10_1" );

	QWidget* wid0 = new QWidget( hbox10_1, "wid0" );
  	wid0->setMinimumWidth(100); //XXX-Davi: Set and use constant

	QVBox* vbox10_10 = new QVBox( hbox10_1, "vbox10_10" );

	QLabel* labelIngredients = new QLabel( "Description   Rate      Cost", vbox10_10);
	labelIngredients->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Normal ) );
	labelIngredients->setGeometry( QRect(360, 80, 410, 30) );

	lbIngredients = new QListBox( vbox10_10, "lbIngredients" );
	lbIngredients->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );
  	lbIngredients->insertItem( QString(MAX__SHORT_DSC) + QString("  ") + QString(MAX_RATE) +
				 QString("  ") + QString(MAX_PRICE__CURRENCY1) );
	//XXX-Davi: XXX-QT/KDE-expert: Be sure that the format of MAX_RATE if fixed
	//XXX-Davi:  Justify to right
  	vbox10_10->setMinimumWidth( lbIngredients->maxItemWidth() + SPACE_TO_SCROLLBAR );
  	vbox10_10->setMaximumWidth( lbIngredients->maxItemWidth() + SPACE_TO_SCROLLBAR );
  	lbIngredients->removeItem( 0 );


	QHBox* hbox10_10_0 = new QHBox( vbox10_10, "hbox10_10_0" );
	hbox10_10_0->setSpacing(SPACING);

	QLabel* labelIngredientsTotal = new QLabel( "Total", hbox10_10_0 );
	labelIngredientsTotal->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	ingredientsTotal = new QLineEdit( hbox10_10_0 );
	ingredientsTotal->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
	ingredientsTotal->setReadOnly(true);
  	ingredientsTotal->setAlignment( Qt::AlignRight );

	QWidget* wSpace10_10_0 = new QWidget( hbox10_10_0, "wSpace10_10_0" );
  	wSpace10_10_0->setMinimumWidth( SPACE_TO_SCROLLBAR - SPACING - 2 );

	QPushButton* ingredientRemove = new QPushButton( "Remove", vbox10_10, "IngredientRemove" );
	ingredientRemove->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );
	ingredientRemove->setGeometry( QRect(360, 285, 390, 30) );
	connect( ingredientRemove, SIGNAL(clicked()), SLOT(remIngredientIfSelected()) );
	//XXX IngredientRemove->setEnabled(false);



	QVBox* vbox01 = new QVBox( this, "vbox01" );
	grid->addWidget( vbox01, 1, 0 ); 

	QHBox* hbox01_0 = new QHBox( vbox01, "hbox01_0" );

	QVBox* vbox01_00 = new QVBox( hbox01_0, "vbox01_00" );
	QVBox* vbox01_01 = new QVBox( hbox01_0, "vbox01_01" );

  	QLabel* labelCod1= new QLabel( "Cod", vbox01_00, "labelCod1" );
  	labelCod1->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	cod1 = new QLineEdit( vbox01_00, "leCod1" );
  	cod1->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
  	cod1->setAlignment( Qt::AlignRight );
	cod1->setValidator(intValidator);
  	connect( cod1, SIGNAL(textChanged(const QString &)), SLOT(selectProdToBuy()) );

  	QLabel* labelDsc1= new QLabel( "Description", vbox01_01, "labelDsc1" );
  	labelDsc1->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	desc1 = new QLineEdit( vbox01_01, "leDesc1" );
  	desc1->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
  	desc1->setReadOnly(true);
      	desc1->setMinimumWidth( 130 + 4 ); //XXX-Davi: No adjust to different fonts
      	desc1->setMaximumWidth( 130 + 4 );
	//desc1->XXX-Davi: set upercase. Mainly to make it easier typing a new product.
  	connect( desc1, SIGNAL(textChanged(const QString &)), SLOT(prodToBuyNotSaved()) );

  	lbProdsToBuy = new QListBox( vbox01, "lbProdsToBuy" );
  	lbProdsToBuy->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );
  	connect( lbProdsToBuy, SIGNAL(currentChanged(QListBoxItem*)), SLOT(prodToBuySelected()) );

	// This method have to be called only once due to it make up 'prodsToBuy = new ProdsToBuy[n+1];'
	// and we delete it into the destructor of this class.
  	prodsToBuy = theShopDB->createProdsToBuy();
  	// We will free the prodsToBuy array at salepoint destruction

	// Get the most long product code
	TheMostLongToBuyProdCode = 0;
  	if( prodsToBuy!=NULL && prodsToBuy[0].shortdsc != NULL )
	{
  		int i;
		for( i=0; prodsToBuy[i].shortdsc != NULL; i++ )
		{
			char* tmp;
			asprintf(&tmp, "%d", prodsToBuy[i].code);
			if ( strlen(tmp) > TheMostLongToBuyProdCode )
				TheMostLongToBuyProdCode = strlen(tmp);
			free(tmp);
  		}

		// Load the list of products to sell into the GUI
  		for( i=0; prodsToBuy[i].shortdsc != NULL; i++ )
  		{
  			char* tmp;
  			asprintf( &tmp, "%d  %s", prodsToBuy[i].code, prodsToBuy[i].shortdsc );

			QString tmp2;
			tmp2="";
			int l;
			l = strlen(tmp);
			int j;
			for( j=0; j+l<TheMostLongToBuyProdCode+2+strlen(MAX__SHORT_DSC); j++ )
			{
				tmp2 = " " + tmp2;
			}
  			lbProdsToBuy->insertItem( tmp2 + QString(tmp) );
  		}
  	}
  	else QMessageBox::warning( 0, "Data Base error",
				   QString("The report_ProdsToBuy() ShopDB method fail. Maybe it \n") +
				   QString("is due to wrong Data Base installation or the table 'products' \n") +
				   QString("is empty. \n\n") + QString(theShopDB->error) );

  	lbProdsToBuy->setMinimumWidth( lbProdsToBuy->maxItemWidth() + SPACE_TO_SCROLLBAR );

	QHBox* hbox01_1 = new QHBox( vbox01, "hbox01_1" );

	prodToBuyNew = new QPushButton( "New", hbox01_1, "prodToBuyNew" );
	prodToBuyNew->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );
  	connect( prodToBuyNew, SIGNAL(clicked()), SLOT(newToBuyProd()) );

	prodToBuyRem = new QPushButton( "Remove", hbox01_1, "prodToBuyRem" );
	prodToBuyRem->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );
  	connect( prodToBuyRem, SIGNAL(clicked()), SLOT(removeToBuyProd()) );

	prodToBuySave = new QPushButton( "Save", hbox01_1, "prodToBuySave" );
	prodToBuySave->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );
	prodToBuySave->setEnabled(false);
  	connect( prodToBuySave, SIGNAL(clicked()), SLOT(saveToBuyProd()) );


	QVBox* vbox11 = new QVBox( this, "vbox11" );
	grid->addWidget( vbox11, 1, 1 ); 

	QHBox* hbox11_0 = new QHBox( vbox11, "hbox11_0" );

	QVBox* vbox11_00 = new QVBox( hbox11_0, "vbox11_00" );
	QVBox* vbox11_01 = new QVBox( hbox11_0, "vbox11_01" );
	QVBox* vbox11_02 = new QVBox( hbox11_0, "vbox11_02" );
	QVBox* vbox11_03 = new QVBox( hbox11_0, "vbox11_03" );

  	QLabel* labelLongDsc1= new QLabel( "Description", vbox11_00, "labelLongDsc1" );
  	labelLongDsc1->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	prodToBuyLongDsc = new QLineEdit( vbox11_00 );
	prodToBuyLongDsc->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	connect( prodToBuyLongDsc, SIGNAL(textChanged(const QString &)), SLOT(prodToBuyNotSaved()) );

  	QLabel* labelUnitCost1= new QLabel( "Unit cost", vbox11_01, "labelUnitCost1" );
  	labelUnitCost1->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	prodToBuyUnitCost = new QLineEdit( vbox11_01 );
	prodToBuyUnitCost->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	prodToBuyUnitCost->setAlignment( Qt::AlignRight );
	// Modify this object. It is not editable. The value is filled form the 'untis' and 'price' fields.

  	QLabel* labelRate1= new QLabel( "Rate", vbox11_02, "labelRate1" );
  	labelRate1->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	rate = new QLineEdit( vbox11_02 );
	rate->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	

  	QLabel* labelRateCost1= new QLabel( "Rate cost", vbox11_03, "labelRateCost1" );
  	labelRateCost1->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	rateCost = new QLineEdit( vbox11_03 );
	rateCost->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	rateCost->setAlignment( Qt::AlignRight );
	// Modify this object. It is not editable. The value is filled form the 'untis' and 'price' fields.

	QHBox* hbox11_1 = new QHBox( vbox11, "hbox11_1" );

	QWidget* wid1 = new QWidget( hbox11_1, "wid1" );
  	wid1->setMinimumWidth(100); //XXX-Davi: Set and use constant

	QPushButton* IngredientAdd = new QPushButton( "Add", hbox11_1, "IngredientAdd" );
	IngredientAdd->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );
	IngredientAdd->setGeometry( QRect(360, 405, 390, 30) );
	connect( IngredientAdd, SIGNAL(clicked()), SLOT(addIngredient()) );

	//XXX-Davi: XXX-QT/KDE-expert: To expand. Does not work. Adjust the strech
	//                             factor and try this idea with the salepoin GUI
	QVBox* vbox11_2 = new QVBox( vbox11, "vbox11_2" );
}

void ProdsManage::remIngredientIfSelected()
{
	if ( lbIngredients->isSelected(lbIngredients->currentItem()) )
	{
		lbIngredients->removeItem( lbIngredients->currentItem() );
		calcTotal();
	}
	//XXX ???  Is the 'current' Item always the 'Selected' item?


	prodToSellNotSaved();
}

void ProdsManage::addIngredient()
{
	// if (all fields has been filled, else warn to the user)  XXX-Davi: ...
	{
		// XXX-Davi: Use a format:  "12345678901  9/999  99999.99"
		lbIngredients->insertItem( desc1->text() + "  " + rate->text() + "  " + rateCost->text() );
		// XXX-Davi: Check that all the field follows the correc 'format' or convert here the format.

		calcTotal();
	}


	prodToSellNotSaved();
}

void ProdsManage::calcTotal()
{
	// Recalculate the total field
	float currency1=0.0;
	unsigned int j;
	for( j=0; j<lbIngredients->count(); j++ )
	{
		currency1 += ( (lbIngredients->text(j)).right(strlen(MAX_PRICE__CURRENCY1)) ).toFloat(); //XXX-Davi: Is this operation accurate?
		// The overflow does not work with this. So, we must
		// find the last word or be sure to avoid the overflow.
		// XXX-Davi: Be sure to avoid the overflow from the cause
	}
	char* tmp;
	asprintf(&tmp, "%.2f", currency1 );
	ingredientsTotal->setText( QString(tmp) );
	free(tmp);
}

void ProdsManage::prodToSellSelected()
{
	if ( prodToSellSave->isEnabled() ) //XXX-Davi: The use of the 'enabled' as mark of a state is 'obscure'
	{
		// Ask to check the lost of modified datas
		int button=-1;
		button = QMessageBox::warning( 0, "Unsaved information",
				       QString("The unsaved data will be lost. Are you sure you want lost it?"),
				       "Accept", "Cancel", QString::null, 1, -1 );
		switch( button )
		{
		case 0: // Accept lost
			break;

		case 1: // Cancel
		default:// Note: This condition raises at least when the window is closed
			//       without click any button => Assuming the default "Cancel"
			return;
			break;
		}
	}

	cod0->setText( ((lbProdsToSell->currentText()).left(TheMostLongToSellProdCode)).stripWhiteSpace() );
	desc0->setText( (lbProdsToSell->currentText()).right(strlen(MAX__SHORT_DSC)) );

	prodToSellLongDsc->setText( QString(prodsToSell[lbProdsToSell->currentItem()].longdsc) );

	char* price;
	unsigned int tmp_price = prodsToSell[lbProdsToSell->currentItem()].price;  //XXX-Davi: Does it fail due to the List Box is now sorted by shordsc. I do not think so due to it is sorted in the query 'ORDER BY'.
	asprintf(&price, "%d.%2d", tmp_price/100, tmp_price%100 ); //XXX-Davi: Use QString
	// XXX-Davi: Think about the '.%2d'. Is the .01 converted to 0.1 by the %100 operation?
	prodToSellUnitPrice->setText(QString(price));
	free(price);


	// Clean the ingredient list of the previous product
	unsigned int i, j;
	j = lbIngredients->count();
	for( i=0; i<j; i++ )
		lbIngredients->removeItem( j -i -1 );

	// Load the ingredient list of this product (if there is)
	   //(XXX-Tomasz): Load the ingredients of the product into the lbIngredients List Box
	   // Use:  lbIngredients->insertItem( QString(tmp) );

	// reCalculate the total field
	calcTotal();


	prodToSellSave->setEnabled(false);
	prodToSellNew->setText("New");
}

void ProdsManage::prodToBuySelected()
{
	cod1->setText( ((lbProdsToBuy->currentText()).left(TheMostLongToBuyProdCode)).stripWhiteSpace() );
	desc1->setText( (lbProdsToBuy->currentText()).right(strlen(MAX__SHORT_DSC)) );

	prodToBuyLongDsc->setText( QString(prodsToBuy[lbProdsToBuy->currentItem()].longdsc) );

	// XXX-Davi
	//  * [davi] [prodsManage window] Automatic calculation of product cost:
	//  	Note: The "balanced mean" will be used as default. It can be changed via
	//  	      configuration to FIFO(deflationary period) or LIFO(inflationary period)  
	//  	1.- If there is product in stock it will be used, if not
	//  	2.- if there is history of purchased products it will be used (only last product), if not
	//  	3.- the field will appear empty
	//  	Note: The user always can edit the field and select a different cost
	//  	Note: The prodsManage GUI can be used to detect lost due to a products is being
	//  	      selling unders its current cost.
	//  	Idea: An option to set automatic "net profit" to each product could be developed.
	//  	      What do you think???
	//  	      Note: The legislation at Spain force to get passed the price with "Tourism",
	//  		    so the automatic "net profic" only work if a "limit" is set.

	// Due to the QLineEdit is not ReadOnly the 'manager' can modify it
	// to correct the suplied by the application.
	prodToBuyUnitCost->setText("");

	rate->setText("");
	rateCost->setText("");


	prodToBuySave->setEnabled(false);
}

void ProdsManage::selectProdToSell()
{
	// If there is, it finds the product with this code into the ListBox and selects it.

	// Make up de 'code' patern
	int i, l;
	QString code;

	code = (cod0->text()).stripWhiteSpace();
	l = code.length();
	for( i=l; i<TheMostLongToSellProdCode; i++ )
		code.insert( 0, QString(" ") );

	// Look for it. Find exact mach of code
	int j;
	j = lbProdsToSell->count();
	for( i=0; i<j && !( (lbProdsToSell->text(i)).left(TheMostLongToSellProdCode) ).contains(code) ; i++ )  ;

	// Select if there is
	if (i<j)
	{
		if( !lbProdsToSell->isSelected(i) )  // To avoid an infinite loop.
			lbProdsToSell->setSelected(i,true);
		lbProdsToSell->ensureCurrentVisible();
	}
}

void ProdsManage::selectProdToBuy()
{
	// If there is, it finds the product with this code into the ListBox and selects it.

	// Make up de 'code' patern
	int i, l;
	QString code;

	code = (cod1->text()).stripWhiteSpace();
	l = code.length();
	for( i=l; i<TheMostLongToBuyProdCode; i++ )
		code.insert( 0, QString(" ") );

	// Look for it. Find exact mach of code
	int j;
	j = lbProdsToBuy->count();
	for( i=0; i<j && !( (lbProdsToBuy->text(i)).left(TheMostLongToBuyProdCode) ).contains(code) ; i++ )  ;

	// Select if there is
	if (i<j)
	{
		if( !lbProdsToBuy->isSelected(i) )  // To avoid an infinite loop.
			lbProdsToBuy->setSelected(i,true);
		lbProdsToBuy->ensureCurrentVisible();
	}
}

void ProdsManage::prodToSellNotSaved()
{
	prodToSellSave->setEnabled(true);
}

void ProdsManage::prodToBuyNotSaved()
{
	prodToBuySave->setEnabled(true);
}

void ProdsManage::newToSellProd()
{
	if ( QString::compare(prodToSellNew->text(),"New")==0 )
	{ // New
		prodToSellNew->setText("Cancel");


		prodToSellRem->setEnabled(false);

		// XXX-Davi: XXX-Tomasz

		// XXX-Davi: Clean the fields and to get ready the window to
		// 'Save' the results as NEW product, not as a UPDATEd product.
		// XXX-Davi: Must we use a flag to mark the product to be updated or added?.
		cod0->setText("-1");
 	 	cod0->setReadOnly(true); // Recover the ReadOnly state after canceling or saving.
		desc0->setText("");
	  	desc0->setReadOnly(false); // Recover the ReadOnly state after canceling or saving.
		prodToSellLongDsc->setText("");
		prodToSellUnitPrice->setText("");
		prodToSellSave->setEnabled(false); // To undo the automatic 'enable' after modification.

		// XXX-Davi: Offer list to recover (recycle code). New widget? Yes.
		//           Maybe, this feature will be added later.
	}else{
	  // Cancel the 'New' action.
		prodToSellNew->setText("New");
		prodToSellRem->setEnabled(true);
		prodToSellSave->setEnabled(false);
	
		cod0->setText(""); // Clean the "-1"
		cod0->setReadOnly(false);
		desc0->setReadOnly(true);

		prodToSellSelected();
	}
}
void ProdsManage::newToBuyProd()
{
	if ( QString::compare(prodToBuyNew->text(),"New")==0 )
	{ // New
		prodToBuyNew->setText("Cancel");


		prodToBuyRem->setEnabled(false);

		// XXX-Davi: XXX-Tomasz

		// XXX-Davi: Clean the fields and to get ready the window to
		// 'Save' the results as NEW product, not as a UPDATEd product.
		// XXX-Davi: Must we use a flag to mark the product to be updated or added?.
		cod1->setText("-1");
 	 	cod1->setReadOnly(true); // Recover the ReadOnly state after canceling or saving.
		desc1->setText("");
	  	desc1->setReadOnly(false); // Recover the ReadOnly state after canceling or saving.
		prodToBuyLongDsc->setText("");
		prodToBuySave->setEnabled(false); // To undo the automatic 'enable' after modification.

		// XXX-Davi: Offer list to recover (recycle code). New widget? Yes.
		//           Maybe, this feature will be added later.
	}else{
	  // Cancel the 'New' action.
		prodToBuyNew->setText("New");
		prodToBuyRem->setEnabled(true);
		prodToBuySave->setEnabled(false);
	
		cod1->setText(""); // Clean the "-1"
		cod1->setReadOnly(false);
		desc1->setReadOnly(true);

		prodToBuySelected();
	}
}

void ProdsManage::removeToSellProd()
{
	prodToSellRem->setEnabled(false);
	prodToSellNew->setEnabled(false);

	int button=-1;
	button = QMessageBox::warning( 0, "Remove",
				       QString("Are you sure that you want unactivate this product?"),
				       "Accept", "Cancel", QString::null, 1, -1 );
	switch( button )
	{
	case 0: // Accept remove 
		//(XXX-Tomasz): Unactivate from the data base. (Notify postgres feature)

		//XXX-Davi:  remake the theShopDB->struct array
			// delete theShopDB;
			// reload;
		//XXX-Davi:  Remake the list box
			// method to reload (reuse centralize code);
		//XXX-Davi:
			 // Comunicate with others GUI; salepoint, buy, ...;

		// XXX-Davi: Will cause trouble 'Sell', 'Buy', 'Edit', 'Remove', due to
		// a application mistake ..., a previously unactivated product?.

		break;

	case 1: // Cancel
	default:// Note: This condition raises at least when the window is closed
		//       without click any button => Assuming the default "Cancel"
		break;
	}

	prodToSellRem->setEnabled(true);
	prodToSellNew->setEnabled(true);
}
void ProdsManage::removeToBuyProd()
{
	prodToBuyRem->setEnabled(false);
	prodToBuyNew->setEnabled(false);

	int button=-1;
	button = QMessageBox::warning( 0, "Remove",
				       QString("Are you sure that you want unactivate this product?"),
				       "Accept", "Cancel", QString::null, 1, -1 );
	switch( button )
	{
	case 0: // Accept remove 
		//(XXX-Tomasz): Unactivate from the data base. (Notify postgres feature)

		//XXX-Davi:  remake the theShopDB->struct array
			// delete theShopDB;
			// reload;
		//XXX-Davi:  Remake the list box
			// method to reload (reuse centralize code);
		//XXX-Davi:
			 // Comunicate with others GUI; salepoint, buy, ...;

		// XXX-Davi: Will cause trouble 'Sell', 'Buy', 'Edit', 'Remove', due to
		// a application mistake ..., a previously unactivated product?.

		break;

	case 1: // Cancel
	default:// Note: This condition raises at least when the window is closed
		//       without click any button => Assuming the default "Cancel"
		break;
	}

	prodToBuyRem->setEnabled(true);
	prodToBuyNew->setEnabled(true);
}

void ProdsManage::saveToSellProd()
{
	// XXX-Davi: The application must guarantee that data are not lost when the user close the window using the window manager button.

	if ( QString::compare(prodToSellNew->text(),"New")==0 ) //XXX-Davi: The use of the caption as mark is 'obscure'
	//XXX-Davi: Use a 'state' private member to manage
	// the close of this window, and ask for save an unsaved state.
	{ // UPDATE product
		// XXX-Tomasz
	}else{
	  // INSERT new product
		// XXX-Tomasz
	}
	// Recover state modified at 'newToSellProd()'
	cod0->setText(""); // Clean the "-1"
	cod0->setReadOnly(false);
	desc0->setReadOnly(true);
	// Manage button state
	prodToSellNew->setText("New");
	prodToSellRem->setEnabled(true);
	// Note: The enabled or no-enabled state show to
	// the user if he/she has modified any data product
	prodToSellSave->setEnabled(false);
}
void ProdsManage::saveToBuyProd()
{
	if ( QString::compare(prodToBuyNew->text(),"New")==0 ) //XXX-Davi: The use of the caption as mark is 'obscure'
	//XXX-Davi: Use a 'state' private member to manage
	// the close of this window, and ask for save an unsaved state.
	{ // UPDATE product
		// XXX-Tomasz
	}else{
	  // INSERT new product
		// XXX-Tomasz
	}
	// Recover state modified at 'newToSellProd()'
	cod1->setText(""); // Clean the "-1"
	cod1->setReadOnly(false);
	desc1->setReadOnly(true);
	// Manage button state
	prodToBuyNew->setText("New");
	prodToBuyRem->setEnabled(true);
	// Note: The enabled or no-enabled state show to
	// the user if he/she has modified any data product
	prodToBuySave->setEnabled(false);
}

ProdsManage::~ProdsManage()
{
	delete prodsToSell;
	delete prodsToBuy;
}
