/***************************************************************************
                          reports.cpp  -  description
                             -------------------
    begin                : Tue Jun 5 2001
    copyright            : (C) 2001 by Jess David Daz Leal
    email                : davileal@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#define _GNU_SOURCE   // This is for asprintf. It has an effect on stdio.h

#include "reports.h"

#include <qlayout.h>
#include <qvbox.h>
#include <qvbuttongroup.h>
#include <qdatetime.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qstatusbar.h>

#include <stdio.h> // To use 'fopen()', ...
#include <stdlib.h> // To use 'system()'


Reports::Reports(QWidget* parent, const char* name) : QWidget( parent, name )
{
	this->setMinimumWidth(780); // XXX-Davi: Does not work, only at init state

	QGridLayout* grid = new QGridLayout( this, 1, 2 );
	grid->setMargin( SPACING );
	grid->setSpacing( SPACING );


	QButtonGroup* reportType = new QButtonGroup( 1, Qt::Horizontal, "Report types", this );
	grid->addWidget( reportType, 0, 0 ); 

	sales = new QRadioButton( "Sales", reportType, "sales" );
	sales->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );	
	connect( sales, SIGNAL(clicked()), SLOT(reportTypeChanged()) );

	purchases = new QRadioButton( "Purchases", reportType, "purchases" );
	purchases->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );	
	connect( purchases, SIGNAL(clicked()), SLOT(reportTypeChanged()) );

	sales_purchases = new QRadioButton( "Sales-Purchases", reportType, "sales_purchases" );
	sales_purchases->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );	
	connect( sales_purchases, SIGNAL(clicked()), SLOT(reportTypeChanged()) );

	current_inventory = new QRadioButton( "Current Inventory", reportType, "current_inventory" );
	current_inventory->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );	
	connect( current_inventory, SIGNAL(clicked()), SLOT(reportTypeChanged()) );

	inventory_deviation = new QRadioButton( "Inventory deviation", reportType, "inventory_deviation" );
	inventory_deviation->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );	
	inventory_deviation->setChecked(true);
	connect( inventory_deviation, SIGNAL(clicked()), SLOT(reportTypeChanged()) );


	QVBox* vbox01 = new QVBox( this, "vbox01" );
	grid->addWidget( vbox01, 0, 1 ); 

	QHBox* hbox01_0 = new QHBox( vbox01, "hbox01_0" );

	QVBox* vbox01_00 = new QVBox( hbox01_0, "vbox01_00" );
	QVBox* vbox01_01 = new QVBox( hbox01_0, "vbox01_01" );

	labelSince = new QLabel( "Since", vbox01_00, "labelSince" );
	labelSince->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	cbSinceDate = new QComboBox( vbox01_00, "cbSinceDate" );
	cbSinceDate->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
	cbSinceDate->insertItem("2001-04-29"); // XXX  This is the list from FixedInventory table
	cbSinceDate->insertItem("2001-07-04");
	cbSinceDate->insertItem("2002-03-13");
	//XXX-Davi: connect( cbSinceDate, SIGNAL(activated(int)), SLOT(updateFileNameText()) );
	//
	leSinceDate = new QLineEdit( vbox01_00, "leSinceDate" );
	leSinceDate->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
	leSinceDate->hide();
	//XXX-Davi: connect( leSinceDate, SIGNAL(clicked()), SLOT(sinceDatePicker()) );

	leSinceTime = new QLineEdit( vbox01_00, "leSinceTime" );
	leSinceTime->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
	leSinceTime->setText("11:30"); //XXX-Davi: Initially empty
	//XXX-Davi: connect( leSinceTime, SIGNAL(activated(int)), SLOT(updateFileNameText()) );


	labelUntil = new QLabel( "Until", vbox01_01, "labelUntil" );
	labelUntil->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

	cbUntilDate = new QComboBox( vbox01_01, "cbUntilDate" );
	cbUntilDate->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
	cbUntilDate->insertItem("2001-04-29"); // XXX  This is the list from FixedInventory table
	cbUntilDate->insertItem("2001-07-04");
	cbUntilDate->insertItem("2002-03-13");
	//XXX-Davi: connect( cbUntilDate, SIGNAL(activated(int)), SLOT(updateFileNameText()) );
	//
	leUntilDate = new QLineEdit( vbox01_01, "leUntilDate" );
	leUntilDate->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
	leUntilDate->hide();
	//XXX-Davi: connect( leUntilDate, SIGNAL(clicked()), SLOT(untilDatePicker()) );

	leUntilTime = new QLineEdit( vbox01_01, "leUntilTime" );
	leUntilTime->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
	leUntilTime->setText("23:59"); //XXX-Davi: Initially empty
	//XXX-Davi: connect( leUntilTime, SIGNAL(activated(int)), SLOT(updateFileNameText()) );


	QWidget* vBoxSpace01_0 = new QWidget( vbox01, "vBoxSpace01_0" );
	vBoxSpace01_0->setMinimumHeight(20);

	//XXX-Davi: At some line must be checked that at least one 'day' is selected.
	weekDays = new QButtonGroup( 1, Qt::Horizontal, "Week days", vbox01 );

	monday = new QCheckBox( "Monday", weekDays, "monday" );
	monday->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );	
	monday->setChecked(true);

	tuesday = new QCheckBox( "Tuesday", weekDays, "tuesday" );
	tuesday->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );	
	tuesday->setChecked(true);

	wednesday = new QCheckBox( "Wednesday", weekDays, "wednesday" );
	wednesday->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );	
	wednesday->setChecked(true);

	thursday = new QCheckBox( "Thursday", weekDays, "thursday" );
	thursday->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );	
	thursday->setChecked(true);

	friday = new QCheckBox( "Friday", weekDays, "friday" );
	friday->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );	
	friday->setChecked(true);

	saturday = new QCheckBox( "Saturday", weekDays, "saturday" );
	saturday->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );	
	saturday->setChecked(true);

	sunday = new QCheckBox( "Sunday", weekDays, "sunday" );
	sunday->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );	
	sunday->setChecked(true);


	QWidget* vBoxSpace01_1 = new QWidget( vbox01, "vBoxSpace01_1" );
	vBoxSpace01_1->setMinimumHeight(20+20);  //XXX-Davi: Must we use constant

	QPushButton* ready = new QPushButton( "Preview Print Save", vbox01, "ready" );
	ready->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );
	connect( ready, SIGNAL(clicked()), SLOT(showTheReport()) );

	QWidget* vBoxSpace01_2 = new QWidget( vbox01, "vBoxSpace01_2" );
	vBoxSpace01_2->setMinimumHeight(20);  //XXX-Davi: Must we use constant


	fileName = new QLineEdit( vbox01 );
	fileName->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
	//XXX-Davi: fileName->setText( "Inventory_deviation--2001-04-29--2002-03-13.html" );

//XXX-Davi, XXX-QT/KDE-expert
//  	selectDate = new KDatePicker( this );
//  	selectDate->setGeometry( QRect(455, 70, 200, 150) );
//  	selectDate->hide();
}

void Reports::reportTypeChanged()
{
	if (sales->isOn())
	{
		labelSince->setEnabled(true);
		labelUntil->setEnabled(true);
		//
		cbSinceDate->hide();
		cbUntilDate->hide();
		//
		leSinceDate->show();
		leUntilDate->show();
		leSinceDate->setEnabled(true);
		leUntilDate->setEnabled(true);

		leSinceTime->setEnabled(true);
		leUntilTime->setEnabled(true);

		weekDays->setEnabled(true);
	}

	if (purchases->isOn())
	{
		labelSince->setEnabled(true);
		labelUntil->setEnabled(true);
		//
		cbSinceDate->hide();
		cbUntilDate->hide();
		//
		leSinceDate->show();
		leUntilDate->show();
		leSinceDate->setEnabled(true);
		leUntilDate->setEnabled(true);

		leSinceTime->setEnabled(true);
		leUntilTime->setEnabled(true);

		weekDays->setEnabled(true);
	}

	if (sales_purchases->isOn())
	{
		labelSince->setEnabled(true);
		labelUntil->setEnabled(true);
		//
		cbSinceDate->show();
		cbUntilDate->show();
		cbSinceDate->setEnabled(true);
		cbUntilDate->setEnabled(true);
		//
		leSinceDate->hide();
		leUntilDate->hide();

		leSinceTime->setEnabled(true);
		leUntilTime->setEnabled(true);

		weekDays->setEnabled(true);
	}

	if (current_inventory->isOn())  // Today
	{
		labelSince->setEnabled(false);
		labelUntil->setEnabled(false);
		//
		cbSinceDate->hide();
		cbUntilDate->hide();
		//
		leSinceDate->show();
		leUntilDate->show();
		leSinceDate->setEnabled(false);
		leUntilDate->setEnabled(false);

		leSinceTime->setEnabled(false);
		leUntilTime->setEnabled(false);

		weekDays->setEnabled(false);
	}

	if (inventory_deviation->isOn())
	{
		labelSince->setEnabled(true);
		labelUntil->setEnabled(true);
		//
		cbSinceDate->show();
		cbUntilDate->show();
		cbSinceDate->setEnabled(true);
		cbUntilDate->setEnabled(true);
		//
		leSinceDate->hide();
		leUntilDate->hide();

		leSinceTime->setEnabled(false);
		leUntilTime->setEnabled(false);

		weekDays->setEnabled(false);
	}

	updateFileNameText();
}

void Reports::updateFileNameText()
{
	fileName->clear();
	if (sales->isOn())               fileName->insert("Sales----------------");
	if (purchases->isOn())           fileName->insert("Purchases------------");
	if (sales_purchases->isOn())     fileName->insert("Sales-Purchases------");
	if (current_inventory->isOn())   fileName->insert("Current_Inventory----");
	if (inventory_deviation->isOn()) fileName->insert("Inventory_deviation--");
	fileName->insert( cbSinceDate->currentText() + "--" + cbUntilDate->currentText() + ".html" );
}

void Reports::showTheReport()
{
	const char* filename = ( (fileName->text()).stripWhiteSpace() ).latin1();

	if( filename!=NULL && *filename!='\0' ) //XXX-Davi: Check that keeps a valid "file name". Is there a predefined function to do this?
	{
		FILE* report;
		report = fopen ( filename, "w+");
		char testXXX[] = "Hi, Hello, Halo!, ..., Bye, Goodbye, I see you later.";
		fwrite( testXXX, sizeof(char), sizeof(char)*strlen(testXXX), report);
		fclose(report);

		char* tmp;
		asprintf(&tmp, "konqueror %s &", filename );
		system(tmp);
		free(tmp);
	}
}

void Reports::sinceDatePicker()
{
// XXX-QT/KDE-expert
//  	if(selectDate->isVisible())
//  	{
//  		selectDate->hide();
//  		QDate Date = selectDate->getDate();
//  		int year = Date.year();
//  		int month = Date.month();
//  		int day = Date.day();
//  		char* date;
//  		sprintf( date, "%d-%d-%d", year, month, day );
//  		pbSinceDate->setText( QString(date) );
//  	}else{
//  		selectDate->show();
//  	}
}

void Reports::untilDatePicker()
{
// XXX-QT/KDE-expert
//  	if(selectDate->isVisible())
//  	{
//  		selectDate->hide();
//  		QDate Date = selectDate->getDate();
//  		int year = Date.year();
//  		int month = Date.month();
//  		int day = Date.day();
//  		char* date;
//  		sprintf( date, "%d-%d-%d", year, month, day );
//  		pbUntilDate->setText( QString(date) );
//  	}else{
//  		selectDate->show();
//  	}
}

Reports::~Reports()
{
}
