/***************************************************************************
                          salepoint.cpp  -  description
                             -------------------
    begin                : Wed May  2 16:34:08 WEST 2001
    copyright            : (C) 2001 by Jess David Daz Leal
    email                : davileal@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#define _GNU_SOURCE   // This is for asprintf. It has an effect on stdio.h

#include "salepoint.h"

#include <qlayout.h>
#include <qvbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qaccel.h>
#include <qmessagebox.h>
#include <qvalidator.h>

#include <stdio.h>

#include <stdlib.h> // To use the 'exit()' function.

#include "login.h"

extern ShopDB* theShopDB;


SalePoint::SalePoint(QWidget* parent, const char* name) : QWidget(parent, name)
{
	QGridLayout* grid = new QGridLayout( this, 1, 3 );
	grid->setMargin( SPACING );
	grid->setSpacing( SPACING );

	QVBox* vbox0 = new QVBox( this, "vbox0" );
	grid->addWidget( vbox0, 0, 0 ); 
	grid->setColStretch( 0, 40 );

  	QLabel* labelSoldProds= new QLabel( "Product          Units      Euros", vbox0, "labelSold" );
  	labelSoldProds->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	lbSoldProds = new QListBox( vbox0, "lbSoldProds" );
  	lbSoldProds->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	lbSoldProds->insertItem( QString(MAX__SHORT_DSC) + QString(" ") + QString(MAX_UNITS) +
				 QString(" ") + QString(MAX_PRICE__CURRENCY1) );
  	lbSoldProds->setMinimumWidth( lbSoldProds->maxItemWidth() + SPACE_TO_SCROLLBAR );
  	lbSoldProds->setMinimumHeight( 420 );
  	lbSoldProds->removeItem( 0 );

	QHBox* hbox0 = new QHBox( vbox0, "hbox0" );
	hbox0->setSpacing(SPACING);

  	QLabel* lbTotal = new QLabel( "Total", hbox0, "lbTotal" );
  	lbTotal->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	totalSold = new QLineEdit( hbox0, "leTotalSold" );
  	totalSold->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	totalSold->setReadOnly(true);
  	totalSold->setAlignment( Qt::AlignRight );

	QWidget* wSpace = new QWidget( hbox0, "wSpace" );
  	wSpace->setMinimumWidth( SPACE_TO_SCROLLBAR - SPACING - 2 );

	QWidget* wSpace2 = new QWidget( vbox0, "wSpaces" );
  	wSpace2->setMinimumHeight(20);

  	QPushButton* remove = new QPushButton( "Remove", vbox0, "pbRemove" );
  	remove->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_3, QFont::Normal ) );
  	connect( remove, SIGNAL(clicked()), SLOT(remIfSelected()) );


	QVBox* vbox1 = new QVBox( this, "vbox1" );
	grid->addWidget( vbox1, 0, 1 ); 
	grid->setColStretch( 1, 30 );

	QHBox* hbox1 = new QHBox( vbox1, "hbox1" );

	QVBox* vbox10 = new QVBox( hbox1, "vbox10" );
	QVBox* vbox11 = new QVBox( hbox1, "vbox11" );

  	QLabel* labelCod= new QLabel( "Cod", vbox10, "labelCod" );
  	labelCod->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	cod = new QLineEdit( vbox10, "leCod" );
  	cod->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
  	cod->setAlignment( Qt::AlignRight );
	QIntValidator* validator = new QIntValidator(this);
	cod->setValidator(validator);
  	connect( cod, SIGNAL(textChanged(const QString &)), SLOT(selectProd()) );

  	QLabel* labelDsc= new QLabel( "Description", vbox11, "labelDsc" );
  	labelDsc->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	desc = new QLineEdit( vbox11, "leDesc" );
  	desc->setFont( QFont( "courier", FONT_SIZE_COURIER_1, QFont::Bold ) );	
  	desc->setReadOnly(true);
      	desc->setMinimumWidth( 130 + 4 ); //XXX-Davi: No adjust to different fonts
      	desc->setMaximumWidth( 130 + 4 );


  	lbProdsToSell = new QListBox( vbox1, "lbProdsToSell" );
  	lbProdsToSell->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );
  	this->setFocusProxy(lbProdsToSell);
  	connect( lbProdsToSell, SIGNAL(currentChanged(QListBoxItem*)), SLOT(prodSelected()) );

	// This method have to be called only once due to it make up 'prodsToSell = new ProdsToSell[n+1];'
	// and we delete it into the destructor of this class.
  	prodsToSell = theShopDB->createProdsToSell();
  	// We will free the prodsToSell array at salepoint destruction

	// Get the most long product code
	TheMostLongProdCode = 0;
  	if( prodsToSell!=NULL && prodsToSell[0].shortdsc != NULL )
	{
  		int i;
		for( i=0; prodsToSell[i].shortdsc != NULL; i++ )
		{
			char* tmp;
			asprintf(&tmp, "%d", prodsToSell[i].code);
			if ( strlen(tmp) > TheMostLongProdCode )
				TheMostLongProdCode = strlen(tmp);
			free(tmp);
  		}

		// Load the list of products to sell into the GUI
  		for( i=0; prodsToSell[i].shortdsc != NULL; i++ )
  		{
  			char* tmp;
  			asprintf( &tmp, "%d  %s", prodsToSell[i].code, prodsToSell[i].shortdsc );

			QString tmp2;
			tmp2="";
			int l;
			l = strlen(tmp);
			int j;
			for( j=0; j+l<TheMostLongProdCode+2+strlen(MAX__SHORT_DSC); j++ )
			{
				tmp2 = " " + tmp2;
			}
  			lbProdsToSell->insertItem( tmp2 + QString(tmp) );
  		}
  	}
  	else QMessageBox::warning( 0, "Data Base error",
				   QString("The report_ProdsToSell() ShopDB method fail. Maybe it \n") +
				   QString("is due to wrong Data Base installation or the table 'products' \n") +
				   QString("is empty. \n\n") + QString(theShopDB->error) );

  	lbProdsToSell->setMinimumWidth( lbProdsToSell->maxItemWidth() + SPACE_TO_SCROLLBAR );


  	// (XXX-Juan)
  	// Client requirement: When a key is pressed the list of products to sell
  	// select and show the first product whose first letter of the 'short description'
  	// is equal to the letter pressed.
  	// NNNote: Not only when lbProdsToSell widget has the focus but when any
  	//         widget is activated.


	const int EDIT=88;
	const int SPIN=61;
	QWidget* wid = new QWidget( this, "wid" );
  	wid->setMinimumWidth(5+SPIN+EDIT+EDIT+5);
	grid->addWidget( wid, 0, 2 ); 
	grid->setColStretch( 2, 30 );

  	QLabel* labelEuro = new QLabel( "Euros", wid, "labelEuro");
	labelEuro->setGeometry( QRect(0+SPIN+SPACING, 0, EDIT, 30) );
  	labelEuro->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );

  	QLabel* labelPtas = new QLabel( "Ptas", wid, "labelPtas");
  	labelPtas->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_2, QFont::Normal ) );
	labelPtas->setGeometry( QRect(0+SPIN+SPACING+EDIT, 0, EDIT, 30) );

  	priceC1 = new QLineEdit( wid, "lePrice" );
  	priceC1->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	priceC1->setReadOnly(true);
  	priceC1->setAlignment( Qt::AlignRight );
	priceC1->setGeometry( QRect(SPIN+SPACING, 40, EDIT, 30) );
  	connect( priceC1, SIGNAL(textChanged(const QString &)), SLOT(reEvaluateTotal()) );

  	priceC2 = new QLineEdit( wid, "lePrice" );
  	priceC2->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	priceC2->setReadOnly(true);
  	priceC2->setAlignment( Qt::AlignRight );
	priceC2->setGeometry( QRect(SPIN+SPACING+EDIT, 40, EDIT, 30) );

  	totalC1 = new QLineEdit( wid, "leTotal" );
  	totalC1->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	totalC1->setReadOnly(true);
  	totalC1->setAlignment( Qt::AlignRight );
  	totalC1->setGeometry( QRect(SPIN+SPACING, 70, EDIT, 30) );

  	totalC2 = new QLineEdit( wid, "leTotal" );
  	totalC2->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	totalC2->setReadOnly(true);
  	totalC2->setAlignment( Qt::AlignRight );
  	totalC2->setGeometry( QRect(SPIN+SPACING+EDIT, 70, EDIT, 30) );

  	units = new QSpinBox( wid, "sbUnits" );
  	units->setFont( QFont( "courier", FONT_SIZE_COURIER_2, QFont::Bold ) );	
  	units->setValue(1);
  	units->setMinValue(1);
  	units->setMaxValue(999);
  	units->setGeometry( QRect(5, 70, SPIN, 30) );
  	connect( units, SIGNAL(valueChanged(int)), SLOT(reEvaluateTotal()) );

  	QPushButton* sold = new QPushButton( "Sold", wid, "pbSold" );
  	sold->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_1, QFont::Normal ) );
  	connect( sold, SIGNAL(clicked()), SLOT(soldProduct()) );
  	sold->setGeometry( QRect(5, 130, SPIN+EDIT+EDIT, 60) );

  	QPushButton* ticket = new QPushButton( "Print ticket", wid, "pbTicket" );
  	ticket->setFont( QFont( "helvetica", FONT_SIZE_HELVETICA_1, QFont::Normal ) );
  	connect( ticket, SIGNAL(clicked()), SLOT(printTicket()) );
  	ticket->setGeometry( QRect(58, 300, 150, 60) );


  	QAccel* accel = new QAccel( this );
  	accel->connectItem( accel->insertItem( Key_Enter ), this, SLOT(soldProduct()) );
  	accel->connectItem( accel->insertItem( Key_Return ),this, SLOT(soldProduct()) );
	//XXX accel->connectItem( accel->insertItem( CTRL+Key_Q ),this, SLOT( hideEvent(QHideEvent*)) );
	//XXX-Davi: The 'Print Screen' key is connected to the 'print tickect' function.
}

void SalePoint::remIfSelected()
{
	if ( lbSoldProds->isSelected(lbSoldProds->currentItem()) )
	{
		lbSoldProds->removeItem( lbSoldProds->currentItem() );
		reEvaluateSoldProdTotal();
	}
}

void SalePoint::soldProduct()
{
	// Add the new row
	int i, l;
	QString tmp, result;

	tmp = desc->text();
	l = tmp.length();
	for( i=l; i<strlen(MAX__SHORT_DSC); i++ )
		tmp.append( QString(" ") );
	result = tmp;

	tmp = units->text();
	l = tmp.length();
	for( i=l; i<strlen(MAX_UNITS); i++ )
		tmp.insert( 0, QString(" ") );
	result = result + " " + tmp;

	tmp = totalC1->text();
	l = tmp.length();
	for( i=l; i<strlen(MAX_PRICE__CURRENCY1); i++ )
		tmp.insert( 0, QString(" ") );
	result = result + " " + tmp;

	lbSoldProds->insertItem( result );

	// Reevaluate the Total
	reEvaluateSoldProdTotal();
}

void SalePoint::reEvaluateSoldProdTotal()
{
	float euros=0.0;
	unsigned int j;
	for( j=0; j<lbSoldProds->count(); j++ )
	{
		euros += ( (lbSoldProds->text(j)).right(strlen(MAX_PRICE__CURRENCY1)) ).toFloat(); //XXX-Davi: Is this operation accuracy?
		// The overflow does not work with this. So, we must
		// find the last word or be sure to avoid the overflow.
		// We now are sure to avoid the overflow from the
		// cause ('totalC1' product total).
	}
	char* tmp;
	asprintf(&tmp, "%.2f", euros );
	totalSold->setText( QString(tmp) );
	free(tmp);
}

void SalePoint::reEvaluateTotal()
{
	unsigned int i_units;

	i_units = (units->text()).toUInt();

	QString qCurrency1 = priceC1->text();
	/*unsigned*/ float f_priceC1;
	f_priceC1 = qCurrency1.toFloat();  //XXX-Davi: Check the accuracy of this operation

	char* Currency1;
	asprintf(&Currency1, "%.2f", i_units*f_priceC1 );
  	// If there is overflow then send a window-warning to the user to call to the support.
	// Check the overflow: Workaround: Avoid the overflow
	if( strlen(Currency1) > strlen(MAX_PRICE__CURRENCY1) ) 
	{
		units->setValue(1);
//XXX-Davi: This is commented to avoid a new bug.
//  		QMessageBox::information( this, "", "Please, split the units to avoid overflow. For example use, 5 and 5 instead of 10." );
//  		//units->stepDown();
  		qWarning("Please, split the units to avoid overflow. For example use, 5 and 5 instead of 10." );
	}else{
		totalC1->setText( QString(Currency1) );
		free(Currency1);

		char* Currency2;
		asprintf(&Currency2, "%d", (int)(i_units*f_priceC1*166.6) );  //XXX-Davi: Check this
		totalC2->setText( QString(Currency2) );
		free(Currency2);
	}
}

void SalePoint::prodSelected()
{
	cod->setText( ((lbProdsToSell->currentText()).left(TheMostLongProdCode)).stripWhiteSpace() );
	desc->setText( (lbProdsToSell->currentText()).right(strlen(MAX__SHORT_DSC)) );

	// Due to the dynamic use of 'price' it is necessary keep in memory the full
	// ProdsToSell view or query each time.
	char* Currency1;
	unsigned int tmp_price = prodsToSell[lbProdsToSell->currentItem()].price;  //XXX-Davi: Does it fail due to the List Box is now sorted by shordsc. I do not think so due to it is sorted in the query 'ORDER BY'.
	asprintf(&Currency1, "%d.%2d", tmp_price/100, tmp_price%100 ); //XXX-Davi: Use QString
		// XXX-Davi: Think about the '.%2d'. Is the .01 converted to 0.1 by the %100 operation?
	priceC1->setText( QString(Currency1) );
	free(Currency1);

	char* Currency2;
	asprintf(&Currency2, "%d", (int)( ((float)tmp_price/100.0)*166.6 ) );  //XXX-Davi: Check this
	priceC2->setText( QString(Currency2) );
	free(Currency2);

	reEvaluateTotal();

	units->setValue(1);
}

void SalePoint::printTicket()
{
	//(XXX-Tomasz)  The products which have been sold are kept in the "PurchasesSalesHistory"
	//              table, filling all the columns of the table. The transactionId is set
	//              in accordance.

	//(XXX-Tomasz)  The products which have been sold are substracted from the "Stock"
	//              in the units according with the units sold and the ingredients of
	//              the "SoldProdsIngredients" table (rate).

	//IIInfo: Although the Stock show 0 (or <0) units of all products
	//        the application can be used to sell. This is to support the case in
	//        which the application's inventory has not been fixed with the
	//        real inventory.
	//        Anyway, it is ridiculous the fact that the waiter want to sell and
	//        the application does not permit it due this issue.

//	Uncomment it XXX-Davi
//	theShopDB->transaction_start();
//	theShopDB->transaction_set_notes("Example note.");
//	theShopDB->sell("WINE T.B.T.", 3); //(XXX-Tomasz): Note: The 'sell' function must substract the Stock units.
//	theShopDB->transaction_end();

	//XXX Print line
	//XXX-Davi: Maybe is not good idea disable the "Print ticket" button so as to
	//    allow to the waiter open the money locker without printing a new ticket
	//    (without the need of selling a new product).

	// Remove all items from lbSoldProds list box
	unsigned int i, j;
	j = lbSoldProds->count();
	for( i=0; i<j; i++ )
		lbSoldProds->removeItem( j -i -1 );
		
	
	reEvaluateSoldProdTotal();
}

void SalePoint::selectProd()
{
	// If there is, it finds the product with this code into the ListBox and selects it.

	// Make up de 'code' patern
	int i, l;
	QString code;

	code = (cod->text()).stripWhiteSpace();
	l = code.length();
	for( i=l; i<TheMostLongProdCode; i++ )
		code.insert( 0, QString(" ") );

	// Look for it. Find exact mach of code
	int j;
	j = lbProdsToSell->count();
	for( i=0; i<j && !( (lbProdsToSell->text(i)).left(TheMostLongProdCode) ).contains(code) ; i++ )  ;

	// Select if there is
	if (i<j)
	{
		if( !lbProdsToSell->isSelected(i) )  // To avoid an infinite loop.
			lbProdsToSell->setSelected(i,true);
		lbProdsToSell->ensureCurrentVisible();
	}
}

SalePoint::~SalePoint()
{
	delete prodsToSell;

	theShopDB->close();
}
