/***************************************************************************
                          shopDB.cpp  -  description
                             -------------------
    begin                : Sun May 13 2001
    copyright            : (C) 2001 by Jess David Daz Leal
    email                : davileal@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define _GNU_SOURCE   // This is for asprintf. It has an effect on stdio.h

#include "shopDB.h"   // This line need to be after _GNU_SOURCE due to it load stdio.h via #include <libpq++.h>

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#define TryRollback(db) \
	if (!db->ExecCommandOk("ROLLBACK")) \
		{ error = "Fatal database error"; return SSERR_FATAL; }

ShopDB::ShopDB()
{
	this->error = NULL;
	this->state = STATE_NOT_LOGIN;
	this->current_transaction_id = -1;
	this->current_transform_id = -1;
	this->appid    = NULL;
	this->username = NULL;
	this->dbname   = "shop";
	this->dbhost   = "localhost";
	this->dbport   = 0;
	this->magick_product = MAGICK_NONE;
}

enum ss_error ShopDB::open()
{
//	parse("/etc/shopsuiterc");
	char* tmp;
	asprintf(&tmp, "%s/.shopsuiterc", getenv("HOME"));
	parse(tmp);
	free(tmp);
//	parse("shopsuiterc");

	if (this->appid == NULL)
		return SSERR_NOAPPID;

	return SSERR_OK;
}

enum ss_error ShopDB::login(const char* passwd)
{
	string dbstring = (string)"dbname=" + this->dbname;
	char* tmp;

	if (this->dbhost)
		dbstring = dbstring + " host=" + this->dbhost;
	if (this->dbport)
	{
		asprintf(&tmp, " port=%d", this->dbport);
		dbstring += tmp;
		free(tmp);
	}
	if (this->username)
		dbstring = dbstring + " user=" + this->username;
	dbstring = dbstring + " password=" + passwd;

	this->db = new PgDatabase(dbstring.data());
	if (this->db->Status() == CONNECTION_BAD)
	{
		asprintf(&this->error, "Can't change transform ingredient: %s", this->db->ErrorMessage());
		delete this->db;
		return SSERR_ERROR;
	}

	this->state = STATE_LOGIN;
	return SSERR_OK;
}

enum ss_error ShopDB::parse(const char* filename)
{
	char* buf1=NULL;
	char* buf2=NULL;
	int n1=0,n2=0;
	int i;
	int fd;
	int l0,l1;

	fd = ::open(filename, O_RDONLY);
	if (fd==-1)
	{
		fprintf(stderr,"\n %s \n", strerror(errno) );
		return SSERR_CANTOPEN;
	}

	buf2 = (char*)malloc(4096); //XXX-Davi: XXX-Tomasz: Why this use of 4096? //XXX-Davi: User <iostream> instead of <stdio.h>
	while ((n2 = read(fd, buf2, 4096)))
	{
		buf1 = (char*)realloc(buf1, n1 + n2);
		memcpy(buf1 + n1, buf2, n2);
		n1 += n2;
	}
	free(buf2);
	::close(fd);

	l0 = l1 = 0;
	for (i=0;i<n1;i++)
	{
		if (buf1[i]=='=')
		    l1=i;
		if (buf1[i]=='\n')
		{
			if (l0 == l1)
			{
				l0 = i;
				continue;
			}
			if (strncmp(buf1+l0, "dbname", 6)==0)
			{
				this->dbname = (char*)malloc(i-l1);
				memcpy(this->dbname, buf1+l1+1, i-l1-1);
				this->dbname [i-l1-1] = 0;
			} else if (strncmp(buf1+l0, "dbhost", 6)==0) {
				this->dbhost = (char*)malloc(i-l1);
				memcpy(this->dbhost, buf1+l1+1, i-l1-1);
				this->dbhost [i-l1-1] = 0;
			} else if (strncmp(buf1+l0, "username", 8)==0) {
				this->username = (char*)malloc(i-l1);
				memcpy(this->username, buf1+l1+1, i-l1-1);
				this->username [i-l1-1] = 0;
			} else if (strncmp(buf1+l0, "appid", 5)==0) {
				this->appid = (char*)malloc(i-l1);
				memcpy(this->appid, buf1+l1+1, i-l1-1);
				this->appid [i-l1-1] = 0;
			} else if (strncmp(buf1+l0, "dbport", 6)==0) {
				this->dbport = atoi(buf1+l1+1);
			} else if (strncmp(buf1+l0, "font", 4) == 0) {
				this->font = (char*) malloc(i - l1);
				memcpy(this->font, buf1 + l1 + 1, i - l1 - 1);
				this->font[i - l1 - 1] = 0;
			}
			l0 = i + 1;
		}
	}
	return SSERR_OK;
}

// should have special error if unit already exists
enum ss_error ShopDB::unit_add(const char* unit)
{
	char* tmp;

	if (this->state != STATE_LOGIN)
	{
		this->error = "Wrong state, can't add unit";
		return SSERR_STATE;
	}
	if (!unit)
		return SSERR_OK;

	asprintf(&tmp, "INSERT INTO measurementunits VALUES ('%s')", unit);
	if (!this->db->ExecCommandOk(tmp))
	{
		free(tmp);
		asprintf(&this->error, "Can't add unit: %s", this->db->ErrorMessage());

		return SSERR_ERROR;
	}
	free(tmp);
	return SSERR_OK;
}

// XXX: should have special error if unit doesn't exist
enum ss_error ShopDB::unit_delete(const char* unit)
{
	char* tmp;

	if (this->state != STATE_LOGIN)
	{
		this->error = "Wrong state, can't delete unit";
		return SSERR_STATE;
	}
	if (!unit)
		return SSERR_OK;

	asprintf(&tmp, "DELETE FROM measurementunits WHERE unit='%s'", unit);
	if (!this->db->ExecCommandOk(tmp))
	{
		free(tmp);
		asprintf(&this->error, "Can't delete unit: %s", this->db->ErrorMessage());
		return SSERR_ERROR;
	}
	free(tmp);
	return SSERR_OK;
}

enum ss_error ShopDB::transform_start(const char* prodid_out, int amountn, int amountd)
{
	char* tmp;

	if (state != STATE_LOGIN)
	{
		this->error = "Wrong state, can't add transform";
		return SSERR_STATE;
	}
	if (!this->db->ExecCommandOk("BEGIN"))
	{
		asprintf(&this->error, "Can't add transform: %s", this->db->ErrorMessage());
		return SSERR_ERROR;
	}
	asprintf(&tmp, "INSERT INTO transform VALUES ((SELECT prodid FROM products WHERE shortdsc='%s'), NULL, TRUE)", prodid_out);
	if (!this->db->ExecCommandOk(tmp))
	{
		free(tmp);
		TryRollback(this->db);
		asprintf(&this->error, "Can't change transform: %s", this->db->ErrorMessage());
		return SSERR_ERROR;
	}
	free(tmp);
	asprintf(&tmp, "SELECT transformid FROM transform WHERE oid = %s", this->db->OidStatus());
	if (!this->db->ExecTuplesOk(tmp))
	{
		free(tmp);
		TryRollback(db);
		asprintf(&this->error, "Can't change transform: %s", this->db->ErrorMessage());
		return SSERR_ERROR;
	}
	free(tmp);
	this->current_transform_id = atoi(this->db->GetValue(0, "transformid"));

	this->tmpn = amountn;
	this->tmpd = amountd;

	this->state = STATE_TRANSFORM;
	return SSERR_OK;
}

enum ss_error ShopDB::transform_ingredient(const char* prodid_in, int amountn, int amountd)
{
	char* tmp;
	(void)amountd;

	if (state != STATE_TRANSFORM)
	{
		this->error = "Wrong state, can't add transform ingredient";
		return SSERR_STATE;
	}
	// requant-on-fly code goes here
	asprintf(&tmp, "INSERT INTO transform_ingredient VALUES (%d,(SELECT prodid FROM products WHERE shortdsc='%s'),%d)",
	this->current_transform_id, prodid_in, amountn);
	if (!this->db->ExecCommandOk(tmp))
	{
		free(tmp);
		TryRollback(db);
		asprintf(&this->error, "Can't change transform ingredient: %s", this->db->ErrorMessage());
		return SSERR_ERROR;
	}
	free(tmp);
	return SSERR_OK;
}

void ShopDB::close()
{
	switch (this->state)
	{
		case STATE_TRANSACTION:
		case STATE_TRANSFORM:
		case STATE_INVENTORYSYNC:
			// rollback
		case STATE_LOGIN:
			delete this->db;
			state = STATE_NOT_LOGIN;
			break;
		case STATE_NOT_LOGIN:
			break;
	}
}

enum ss_error ShopDB::product_new(const char* shortdsc, const char* longdsc, const char* units)
{
	char* tmp;
	asprintf(&tmp, "INSERT INTO products VALUES ('%s', '%s', '%s', NULL, 1, 0, TRUE)",
		 shortdsc, longdsc, units);
	if (!this->db->ExecCommandOk(tmp))
	{
		free(tmp);
		asprintf(&this->error, "Can't add product: %s", this->db->ErrorMessage());
		return SSERR_ERROR;
	}
	return SSERR_OK;
}

enum ss_error ShopDB::product_stock_set(const char* prod, int stockn, int stockd)
{
	char* tmp;
	(void)stockd;
	// requant-on-fly
	asprintf(&tmp, "UPDATE products SET stock=%d WHERE shortdsc='%s'",
		 stockn, prod);
	if (!this->db->ExecCommandOk(tmp))
	{
		free(tmp);
		asprintf(&this->error, "Can't set stock: %s", this->db->ErrorMessage());
		return SSERR_ERROR;
	}
	free(tmp);
	return SSERR_OK;
}

enum ss_error ShopDB::product_price_set(const char* prod, int price)
{
	char* tmp;
	// any requanting required ?
	asprintf(&tmp, "UPDATE products SET sellprice=%d.%d WHERE shortdsc='%s'",
		 price / 100, price % 100, prod);
	if (!this->db->ExecCommandOk(tmp))
	{
		free(tmp);
		asprintf(&this->error, "Can't change price: %s", this->db->ErrorMessage());
		return SSERR_ERROR;
	}
	free(tmp);
	return SSERR_OK;
}

//bool ShopDB::product_is_active(const char* prod)
//{
//	char* tmp;
//	asprintf(&tmp, "SELECT active FROM products WHERE shortdsc='%s'", prod);
//}

enum ss_error ShopDB::product_set_active(const char* prod, bool active)
{
	char* tmp;

	asprintf(&tmp, "UPDATE products SET active=%s WHERE shortdsc='%s'",
	active?"TRUE":"FALSE", prod);
	if (!this->db->ExecCommandOk(tmp))
	{
		free(tmp);
		asprintf(&this->error, "Can't change activity: %s", this->db->ErrorMessage());
		return SSERR_ERROR;
	}
	free(tmp);
	return SSERR_OK;
}

enum ss_error ShopDB::transform_end()
{
	if (this->state != STATE_TRANSFORM)
	{
		this->error = "Wrong state, can't finish adding transform";
		return SSERR_STATE;
	}
	this->state = STATE_LOGIN;
	this->current_transform_id = -1;
	if (!this->db->ExecCommandOk("COMMIT"))
		return SSERR_ERROR;
	return SSERR_OK;
}

char** ShopDB::report_units()
{
	char** units;
	int i,n;

	if (!this->db->ExecTuplesOk("SELECT unit FROM measurementunits"))
	{
		asprintf(&this->error, "Can't get units list: %s", this->db->ErrorMessage());
		return NULL;
	}
	n = this->db->Tuples();
	units = (char**) malloc(sizeof(char*) * (n+1));
	for (i=0; i<n; i++)
		units[i] = strdup(this->db->GetValue (i, "unit"));
	units[n] = NULL;
	return units;
}

//XXX-Davi  Maybe the login window could ask for the login name, db name, etc, beside the password.
//          I do not know now if it could be good idea. I will think about it later.

//XXX-Davi  Check the support of wrong format into the shopsuiterc file.

const ProdsToSell* ShopDB::createProdsToSell()
{
	ProdsToSell* prods;
	int i,n;

	if (!this->db->ExecTuplesOk("SELECT shortdsc,longdsc,units,int8(sellprice*100) as price,prodid FROM ProdsToSell ORDER BY shortdsc"))
	{
		asprintf(&this->error, "Can't get products to sell list: %s", this->db->ErrorMessage());
		return NULL;
	}
	n = this->db->Tuples();

	prods = new ProdsToSell[n+1];
	for (i=0; i<n; i++)
	{
		prods[i].code     = atoi   (db->GetValue (i,"prodid"));
		prods[i].shortdsc = strdup (db->GetValue (i,"shortdsc")); //XXX-Davi: Warning: limit to 11 USE CONSTANT.  What happens if the DB keeps a > 11 ? Is the chars not showed? Must the DB be 11 chars?
		prods[i].longdsc  = strdup (db->GetValue (i,"longdsc"));
		prods[i].unit     = strdup (db->GetValue (i,"units"));
		prods[i].price    = atoi   (db->GetValue (i,"price"));
	}
	prods[n].code     = 0;
	prods[n].shortdsc = NULL;
	prods[n].longdsc  = NULL;
	prods[n].unit     = NULL;
	prods[n].price    = 0;

	return prods;
}

const ProdsToBuy* ShopDB::createProdsToBuy()
{
	ProdsToBuy* prods;
	int i,n;

	if (!this->db->ExecTuplesOk("SELECT shortdsc,longdsc,units,stock,prodid FROM ProdsToBuy ORDER BY shortdsc"))
	{
		asprintf(&this->error, "Can't get products to buy list: %s", this->db->ErrorMessage());
		return NULL;
	}
	n = this->db->Tuples();

	prods = new ProdsToBuy[n+1];
	for (i=0; i<n; i++)
	{
		prods[i].code     = atoi   (db->GetValue (i,"prodid"));
		prods[i].shortdsc = strdup (db->GetValue (i,"shortdsc")); //XXX-Davi: Warning: limit to 11 USE CONSTANT.  What happens if the DB keeps a > 11 ? Is the chars not showed? Must the DB be 11 chars?
		prods[i].longdsc  = strdup (db->GetValue (i,"longdsc"));
		prods[i].unit     = strdup (db->GetValue (i,"units"));
		prods[i].stock    = atoi   (db->GetValue (i,"stock"));
	}
	prods[n].code     = 0;
	prods[n].shortdsc = NULL;
	prods[n].longdsc  = NULL;
	prods[n].unit     = NULL;
	prods[n].stock    = 0;

	return prods;
}

enum ss_error ShopDB::transform_current_set_notes(char* text)
{
	char* tmp;
	asprintf(&tmp, "UPDATE transform SET notes='%s' WHERE transformid=%d",
		 text, this->current_transform_id);

	if (!this->db->ExecCommandOk(tmp))
	{
		free(tmp);
		asprintf(&this->error, "Can't set transform notes: %s", this->db->ErrorMessage());
		return SSERR_ERROR;
	}
	free(tmp);
	return SSERR_OK;
}

ShopDB::~ShopDB()
{
}
