/***************************************************************************
                          shopDB.h  -  description
                             -------------------
    begin                : Sun May 13 2001
    copyright            : (C) 2001 by Jess David Daz Leal
    email                : davileal@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef SHOPDB_H
#define SHOPDB_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <libpq++.h>

enum shopdb_state
{
	STATE_NOT_LOGIN,
	STATE_LOGIN,
	STATE_TRANSACTION,
	STATE_TRANSFORM,
	STATE_INVENTORYSYNC
};

enum ss_error
{
	SSERR_OK,
	SSERR_NOAPPID,
	SSERR_CANTOPEN,
	SSERR_STATE,
	SSERR_ERROR,
	SSERR_FATAL
};

class ProdsToSell
{
	public:
		unsigned int code;
		char* shortdsc;
		char* longdsc;
		Money price;
		char* unit; // Measurement unit
};

class ProdsToBuy
{
	public:
		unsigned int code;
		char* shortdsc;
		char* longdsc;
		char* unit; // Measurement unit
		int stock;
};

enum magick_product
{
	MAGICK_NONE,
	MAGICK_PRODUCT,
	MAGICK_TRANSACTION,
	MAGICK_ALWAYS
};

class ShopDB
{
	public:
		ShopDB();
		~ShopDB();
		
		enum ss_error open();
		void close();
		enum ss_error login(const char* passwd);

		enum ss_error unit_add(const char* unit);
		enum ss_error unit_delete(const char* unit);

		char** report_units();
		const ProdsToSell* createProdsToSell();
		const ProdsToBuy*  createProdsToBuy();

		enum ss_error product_new(const char* shortdsc, const char* longdsc, const char* units);
		enum ss_error product_stock_set(const char* prod, int stockn, int stockd);
		enum ss_error product_price_set(const char* prod, int price);
		bool product_is_active(const char* prod);
		enum ss_error product_set_active(const char* prod, bool active);

		enum ss_error transform_start(const char* prod_out, int amountn, int amountd);
		enum ss_error transform_ingredient(const char* prod_in, int amountn, int amountd);
		enum ss_error transform_end();
		enum ss_error transform_current_set_notes(char* text);

		enum ss_error transaction_start();
		enum ss_error transaction_current_set_notes(char* text);
		enum ss_error transaction_end();
		enum ss_error sell(char* prod, int amountn, int amountd);
		enum ss_error purchase(char* prod, int amountn, int amountd, int price);

		enum ss_error deviationtype_new(char* shortdsc, char* longdsc);
		enum ss_error inventorysync_start();
		enum ss_error inventorysync_current_set_notes(char* text);
		enum ss_error inventorysync_end();
		enum ss_error deviation(char* prod, char* devtype, int amountn, int amountd);

		enum ss_error solve_recursion();

		char* error;
		char* appid;
		char* font;
		char* fontsize;
		// XXXXXX-Davi
		// -b&h-lucidatypewriter-medium-r-normal-*-*-180-*-*-m-*-iso8859-1
		// We will only support some fields: font=lucidatypewriter, fontsize=18
		
		enum shopdb_state get_state(){ return state; }

	private:
		enum ss_error parse(const char* filename);

		PgDatabase* db;
		int current_transaction_id;
		int current_transform_id;
		char* username;
		char* dbname;
		char* dbhost;
		int   dbport;
		enum shopdb_state state;
		enum magick_product magick_product;
		int tmpn, tmpd;
};

#endif
